
#include "comb/catalan-gray.h"

#include "bits/print-bin.h"
#include "comb/comb-print.h"


#include "fxtiomanip.h"
#include "fxttypes.h"
#include "jjassert.h"

#include "nextarg.h"


//% Catalan restricted growth strings (RGS) and parentheses strings
// in minimal-change order (Gray code).

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 5;
    NXARG(n, "Number of paren pairs");
    bool dr0 = true;
    NXARG(dr0, "Starting direction in recursion");

    catalan_gray C(n);
    C.first(dr0);

    ulong ct = 0;

#ifdef TIMING
//    bool bw = 0;
//    NXARG(bw, "Whether to generate backward order");
//
//    if ( bw )
//    {
//        C.last();
//        do  { ++ct; }  while ( C.prev() );
//    }
//    else
    {
        C.first(dr0);
        do  { ++ct; }  while ( C.next() );
    }
#else

//    C.last();

    const ulong n2 = 2*n;
    ulong j = 0;
    do
    {
        ++ct;
        cout << setw(4) << ct << ":  ";

        // print RGS:
        print_vec("  ", C.data(), n, true );

        cout << setw(4) << j;

        // print paren string:
        const char *str = C.str();
        cout << "    " << str;

        // print delta set:
        cout << "    ";
        for (ulong k=0; k<n2; ++k)  cout << ( str[k]=='(' ? '1' : '.' );


        ulong w2 = 0;  // bit-string representation
        for (ulong k=0,m=(1UL<<(n2-1)); k<n2; ++k,m>>=1)  if ( str[k]=='(' )  w2 |= m;

        cout << endl;
    }
    while ( (j=C.next()) );
//    while ( (j=C.prev()) );

#endif  // TIMING

    cout << " ct=" << ct;  // Catalan number n
    cout << endl;

    return 0;
}
// -------------------------


/*
Timing: (AMD Phenom II X4 945 3000MHz)

 time ./bin 18 0
arg 1: 18 == n  [Number of paren pairs]  default=5
arg 2: 0 == dr0  [Starting direction in recursion]  default=1
 ct=477638700
./bin 18 0  2.98s user 0.00s system 99% cpu 2.985 total
 ==> 477638700/2.98 == 160,281,442 per second

 time ./bin 18 1
same (as the very same update step is used)

*/


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/catalan-gray-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/catalan-gray-demo.cc DEMOFLAGS=-DTIMING"
/// End:

